@@ -24,6 +24,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "expression.h"
 #include "target.h"
 #include "frame.h"
+#include "language.h"	/* For CAST_IS_CONVERSION */
 
 /* Values of NOSIDE argument to eval_subexp.  */
 enum noside
@@ -161,7 +162,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
      enum noside noside;
 {
   enum exp_opcode op;
-  int tem;
+  int tem, tem2, tem3;
   register int pc, pc2, oldpos;
   register value arg1, arg2, arg3;
   struct type *type;
@@ -174,22 +175,21 @@ evaluate_subexp (expect_type, exp, pos, noside)
   switch (op)
     {
     case OP_SCOPE:
-      tem = strlen (&exp->elts[pc + 2].string);
-      (*pos) += 3 + ((tem + sizeof (union exp_element))
-		     / sizeof (union exp_element));
+      tem = longest_to_int (exp->elts[pc + 2].longconst);
+      (*pos) += 4 + BYTES_TO_EXP_ELEM (tem + 1);
       arg1 = value_struct_elt_for_reference (exp->elts[pc + 1].type,
 					     0,
 					     exp->elts[pc + 1].type,
-					     &exp->elts[pc + 2].string,
+					     &exp->elts[pc + 3].string,
 					     expect_type);
       if (arg1 == NULL)
-	error ("There is no field named %s", &exp->elts[pc + 2].string);
+	error ("There is no field named %s", &exp->elts[pc + 3].string);
       return arg1;
 
     case OP_LONG:
       (*pos) += 3;
       return value_from_longest (exp->elts[pc + 1].type,
-			      exp->elts[pc + 2].longconst);
+				 exp->elts[pc + 2].longconst);
 
     case OP_DOUBLE:
       (*pos) += 3;
@@ -237,22 +237,42 @@ evaluate_subexp (expect_type, exp, pos, noside)
       (*pos) += 2;
       return value_of_register (longest_to_int (exp->elts[pc + 1].longconst));
 
+
     case OP_INTERNALVAR:
       (*pos) += 2;
       return value_of_internalvar (exp->elts[pc + 1].internalvar);
 
     case OP_STRING:
-      tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + ((tem + sizeof (union exp_element))
-		     / sizeof (union exp_element));
+      tem = longest_to_int (exp->elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (tem + 1);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_string (&exp->elts[pc + 1].string, tem);
+      return value_string (&exp->elts[pc + 2].string, tem);
+
+    case OP_BITSTRING:
+      error ("support for OP_BITSTRING unimplemented");
+      break;
+
+    case OP_ARRAY:
+      (*pos) += 3;
+      tem2 = longest_to_int (exp->elts[pc + 1].longconst);
+      tem3 = longest_to_int (exp->elts[pc + 2].longconst);
+      nargs = tem3 - tem2 + 1;
+      argvec = (value *) alloca (sizeof (value) * nargs);
+      for (tem = 0; tem < nargs; tem++)
+	{
+	  /* Ensure that array expressions are coerced into pointer objects. */
+	  argvec[tem] = evaluate_subexp_with_coercion (exp, pos, noside);
+	}
+      if (noside == EVAL_SKIP)
+	goto nosideret;
+      return (value_array (tem2, tem3, argvec));
+      break;
 
     case TERNOP_COND:
       /* Skip third and second args to evaluate the first one.  */
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
-      if (value_zerop (arg1))
+      if (value_logical_not (arg1))
 	{
 	  evaluate_subexp (NULL_TYPE, exp, pos, EVAL_SKIP);
 	  return evaluate_subexp (NULL_TYPE, exp, pos, noside);
@@ -342,8 +362,8 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	  nargs = longest_to_int (exp->elts[pc + 1].longconst) + 1;
 	  /* First, evaluate the structure into arg2 */
 	  pc2 = (*pos)++;
-	  tem2 = strlen (&exp->elts[pc2 + 1].string);
-	  *pos += 2 + (tem2 + sizeof (union exp_element)) / sizeof (union exp_element);
+	  tem2 = longest_to_int (exp->elts[pc2 + 1].longconst);
+	  *pos += 3 + BYTES_TO_EXP_ELEM (tem2 + 1);
 	  if (noside == EVAL_SKIP)
 	    goto nosideret;
 
@@ -363,6 +383,8 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	  nargs = longest_to_int (exp->elts[pc + 1].longconst);
 	  tem = 0;
 	}
+      /* Allocate arg vector, including space for the function to be
+	 called in argvec[0] and a terminating NULL */
       argvec = (value *) alloca (sizeof (value) * (nargs + 2));
       for (; tem <= nargs; tem++)
 	/* Ensure that array expressions are coerced into pointer objects. */
@@ -378,14 +400,14 @@ evaluate_subexp (expect_type, exp, pos, noside)
 
 	  argvec[1] = arg2;
 	  argvec[0] =
-	    value_struct_elt (&temp, argvec+1, &exp->elts[pc2 + 1].string,
+	    value_struct_elt (&temp, argvec+1, &exp->elts[pc2 + 2].string,
 			      &static_memfuncp,
 			      op == STRUCTOP_STRUCT
 			      ? "structure" : "structure pointer");
 	  if (VALUE_OFFSET (temp))
 	    {
 	      arg2 = value_from_longest (lookup_pointer_type (VALUE_TYPE (temp)),
-				      value_as_long (arg2)+VALUE_OFFSET (temp));
+					 VALUE_ADDRESS (temp)+VALUE_OFFSET (temp));
 	      argvec[1] = arg2;
 	    }
 	  if (static_memfuncp)
@@ -423,40 +445,39 @@ evaluate_subexp (expect_type, exp, pos, noside)
       return call_function_by_hand (argvec[0], nargs, argvec + 1);
 
     case STRUCTOP_STRUCT:
-      tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + ((tem + sizeof (union exp_element))
-		     / sizeof (union exp_element));
+      tem = longest_to_int (exp->elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (tem + 1);
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (noside == EVAL_AVOID_SIDE_EFFECTS)
 	return value_zero (lookup_struct_elt_type (VALUE_TYPE (arg1),
-						   &exp->elts[pc + 1].string,
+						   &exp->elts[pc + 2].string,
 						   0),
 			   lval_memory);
       else
 	{
 	  value temp = arg1;
-	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 1].string,
+	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 2].string,
 				   (int *) 0, "structure");
 	}
 
     case STRUCTOP_PTR:
-      tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+      tem = longest_to_int (exp->elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (tem + 1);
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (noside == EVAL_AVOID_SIDE_EFFECTS)
 	return value_zero (lookup_struct_elt_type (TYPE_TARGET_TYPE
 						   (VALUE_TYPE (arg1)),
-						   &exp->elts[pc + 1].string,
+						   &exp->elts[pc + 2].string,
 						   0),
 			   lval_memory);
       else
 	{
 	  value temp = arg1;
-	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 1].string,
+	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 2].string,
 				   (int *) 0, "structure pointer");
 	}
 
@@ -485,6 +506,16 @@ evaluate_subexp (expect_type, exp, pos, noside)
     bad_pointer_to_member:
       error("non-pointer-to-member value used in pointer-to-member construct");
 
+    case BINOP_CONCAT:
+      arg1 = evaluate_subexp_with_coercion (exp, pos, noside);
+      arg2 = evaluate_subexp_with_coercion (exp, pos, noside);
+      if (noside == EVAL_SKIP)
+	goto nosideret;
+      if (binop_user_defined_p (op, arg1, arg2))
+	return value_x_binop (arg1, arg2, op, OP_NULL);
+      else
+	return value_concat (arg1, arg2);
+
     case BINOP_ASSIGN:
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
@@ -535,11 +566,12 @@ evaluate_subexp (expect_type, exp, pos, noside)
     case BINOP_MUL:
     case BINOP_DIV:
     case BINOP_REM:
+    case BINOP_MOD:
     case BINOP_LSH:
     case BINOP_RSH:
-    case BINOP_LOGAND:
-    case BINOP_LOGIOR:
-    case BINOP_LOGXOR:
+    case BINOP_BITWISE_AND:
+    case BINOP_BITWISE_IOR:
+    case BINOP_BITWISE_XOR:
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
@@ -548,7 +580,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return value_x_binop (arg1, arg2, op, OP_NULL);
       else
 	if (noside == EVAL_AVOID_SIDE_EFFECTS
-	    && (op == BINOP_DIV || op == BINOP_REM))
+	    && (op == BINOP_DIV || op == BINOP_REM || op == BINOP_MOD))
 	  return value_zero (VALUE_TYPE (arg1), not_lval);
       else
 	return value_binop (arg1, arg2, op);
@@ -577,7 +609,58 @@ evaluate_subexp (expect_type, exp, pos, noside)
       else
 	return value_subscript (arg1, arg2);
       
-    case BINOP_AND:
+    case MULTI_SUBSCRIPT:
+      (*pos) += 2;
+      nargs = longest_to_int (exp->elts[pc + 1].longconst);
+      arg1 = evaluate_subexp_with_coercion (exp, pos, noside);
+      while (nargs-- > 0)
+	{
+	  arg2 = evaluate_subexp_with_coercion (exp, pos, noside);
+	  /* FIXME:  EVAL_SKIP handling may not be correct. */
+	  if (noside == EVAL_SKIP)
+	    {
+	      if (nargs > 0)
+		{
+		  continue;
+		}
+	      else
+		{
+		  goto nosideret;
+		}
+	    }
+	  /* FIXME:  EVAL_AVOID_SIDE_EFFECTS handling may not be correct. */
+	  if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	    {
+	      /* If the user attempts to subscript something that has no target
+		 type (like a plain int variable for example), then report this
+		 as an error. */
+	      
+	      type = TYPE_TARGET_TYPE (VALUE_TYPE (arg1));
+	      if (type != NULL)
+		{
+		  arg1 = value_zero (type, VALUE_LVAL (arg1));
+		  noside = EVAL_SKIP;
+		  continue;
+		}
+	      else
+		{
+		  error ("cannot subscript something of type `%s'",
+			 TYPE_NAME (VALUE_TYPE (arg1)));
+		}
+	    }
+	  
+	  if (binop_user_defined_p (op, arg1, arg2))
+	    {
+	      arg1 = value_x_binop (arg1, arg2, op, OP_NULL);
+	    }
+	  else
+	    {
+	      arg1 = value_subscript (arg1, arg2);
+	    }
+	}
+      return (arg1);
+
+    case BINOP_LOGICAL_AND:
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	{
@@ -596,14 +679,14 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
       else
 	{
-	  tem = value_zerop (arg1);
+	  tem = value_logical_not (arg1);
 	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos,
 				  (tem ? EVAL_SKIP : noside));
 	  return value_from_longest (builtin_type_int,
-				  (LONGEST) (!tem && !value_zerop (arg2)));
+				  (LONGEST) (!tem && !value_logical_not (arg2)));
 	}
 
-    case BINOP_OR:
+    case BINOP_LOGICAL_OR:
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	{
@@ -622,11 +705,11 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
       else
 	{
-	  tem = value_zerop (arg1);
+	  tem = value_logical_not (arg1);
 	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos,
 				  (!tem ? EVAL_SKIP : noside));
 	  return value_from_longest (builtin_type_int,
-				  (LONGEST) (!tem || !value_zerop (arg2)));
+				  (LONGEST) (!tem || !value_logical_not (arg2)));
 	}
 
     case BINOP_EQUAL:
@@ -745,19 +828,19 @@ evaluate_subexp (expect_type, exp, pos, noside)
       else
 	return value_neg (arg1);
 
-    case UNOP_LOGNOT:
+    case UNOP_COMPLEMENT:
       /* C++: check for and handle destructor names.  */
       op = exp->elts[*pos].opcode;
 
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      if (unop_user_defined_p (UNOP_LOGNOT, arg1))
-	return value_x_unop (arg1, UNOP_LOGNOT);
+      if (unop_user_defined_p (UNOP_COMPLEMENT, arg1))
+	return value_x_unop (arg1, UNOP_COMPLEMENT);
       else
-	return value_lognot (arg1);
+	return value_complement (arg1);
 
-    case UNOP_ZEROP:
+    case UNOP_LOGICAL_NOT:
       arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -765,7 +848,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return value_x_unop (arg1, op);
       else
 	return value_from_longest (builtin_type_int,
-				(LONGEST) value_zerop (arg1));
+				   (LONGEST) value_logical_not (arg1));
 
     case UNOP_IND:
       if (expect_type && TYPE_CODE (expect_type) == TYPE_CODE_PTR)
@@ -799,9 +882,8 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	{
 	  if (op == OP_SCOPE)
 	    {
-	      char *name = &exp->elts[pc+3].string;
-	      int temm = strlen (name);
-	      (*pos) += 2 + (temm + sizeof (union exp_element)) / sizeof (union exp_element);
+	      int temm = longest_to_int (exp->elts[pc+3].longconst);
+	      (*pos) += 3 + BYTES_TO_EXP_ELEM (temm + 1);
 	    }
 	  else
 	    evaluate_subexp (expect_type, exp, pos, EVAL_SKIP);
@@ -984,9 +1066,16 @@ evaluate_subexp_for_address (exp, pos, noside)
 }
 
 /* Evaluate like `evaluate_subexp' except coercing arrays to pointers.
-   When used in contexts where arrays will be coerced anyway,
-   this is equivalent to `evaluate_subexp'
-   but much faster because it avoids actually fetching array contents.  */
+   When used in contexts where arrays will be coerced anyway, this is
+   equivalent to `evaluate_subexp' but much faster because it avoids
+   actually fetching array contents.
+
+   Note that we currently only do the coercion for C expressions, where
+   arrays are zero based and the coercion is correct.  For other languages,
+   with nonzero based arrays, coercion loses.  Use CAST_IS_CONVERSION
+   to decide if coercion is appropriate.
+
+  */
 
 static value
 evaluate_subexp_with_coercion (exp, pos, noside)
@@ -1006,7 +1095,8 @@ evaluate_subexp_with_coercion (exp, pos, noside)
     {
     case OP_VAR_VALUE:
       var = exp->elts[pc + 1].symbol;
-      if (TYPE_CODE (SYMBOL_TYPE (var)) == TYPE_CODE_ARRAY)
+      if (TYPE_CODE (SYMBOL_TYPE (var)) == TYPE_CODE_ARRAY
+	  && CAST_IS_CONVERSION)
 	{
 	  (*pos) += 3;
 	  val = locate_var_value (var, (FRAME) 0);
