@@ -65,6 +65,7 @@ print_subexp (exp, pos, stream, prec)
   /* Set to 1 for a right-associative operator.  */
   int assoc;
   value val;
+  char *tempstr;
 
   op_print_tab = exp->language_defn->la_op_print_tab;
   pc = (*pos)++;
@@ -76,20 +77,19 @@ print_subexp (exp, pos, stream, prec)
     case OP_SCOPE:
       myprec = PREC_PREFIX;
       assoc = 0;
-      (*pos) += 2;
+      (*pos) += 3;
       print_subexp (exp, pos, stream,
 		    (enum precedence) ((int) myprec + assoc));
       fputs_filtered (" :: ", stream);
-      nargs = strlen (&exp->elts[pc + 2].string);
-      (*pos) += 1 + (nargs + sizeof (union exp_element)) / sizeof (union exp_element);
-
-      fputs_filtered (&exp->elts[pc + 2].string, stream);
+      nargs = longest_to_int (exp->elts[pc + 2].longconst);
+      (*pos) += 2 + BYTES_TO_EXP_ELEM (nargs + 1);
+      fputs_filtered (&exp->elts[pc + 3].string, stream);
       return;
 
     case OP_LONG:
       (*pos) += 3;
       value_print (value_from_longest (exp->elts[pc + 1].type,
-				    exp->elts[pc + 2].longconst),
+				       exp->elts[pc + 2].longconst),
 		   stream, 0, Val_no_prettyprint);
       return;
 
@@ -102,7 +102,7 @@ print_subexp (exp, pos, stream, prec)
 
     case OP_VAR_VALUE:
       (*pos) += 2;
-      fputs_filtered (SYMBOL_NAME (exp->elts[pc + 1].symbol), stream);
+      fputs_filtered (SYMBOL_SOURCE_NAME (exp->elts[pc + 1].symbol), stream);
       return;
 
     case OP_LAST:
@@ -117,6 +117,13 @@ print_subexp (exp, pos, stream, prec)
 	       reg_names[longest_to_int (exp->elts[pc + 1].longconst)]);
       return;
 
+    case OP_BOOL:
+      (*pos) += 2;
+      fprintf_filtered (stream, "%s",
+			longest_to_int (exp->elts[pc + 1].longconst)
+			? "TRUE" : "FALSE");
+      return;
+
     case OP_INTERNALVAR:
       (*pos) += 2;
       fprintf_filtered (stream, "$%s",
@@ -138,12 +145,72 @@ print_subexp (exp, pos, stream, prec)
       return;
 
     case OP_STRING:
-      nargs = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (nargs + sizeof (union exp_element)) / sizeof (union exp_element);
-      fputs_filtered ("\"", stream);
-      for (tem = 0; tem < nargs; tem++)
-	printchar ((&exp->elts[pc + 1].string)[tem], stream, '"');
-      fputs_filtered ("\"", stream);
+      nargs = longest_to_int (exp -> elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (nargs + 1);
+      /* LA_PRINT_STRING will print using the current repeat count threshold.
+	 If necessary, we can temporarily set it to zero, or pass it as an
+	 additional parameter to LA_PRINT_STRING.  -fnf */
+      LA_PRINT_STRING (stream, &exp->elts[pc + 2].string, nargs, 0);
+      return;
+
+    case OP_BITSTRING:
+      error ("support for OP_BITSTRING unimplemented");
+      break;
+
+    case OP_ARRAY:
+      (*pos) += 3;
+      nargs = longest_to_int (exp->elts[pc + 2].longconst);
+      nargs -= longest_to_int (exp->elts[pc + 1].longconst);
+      nargs++;
+      tem = 0;
+      if (exp->elts[pc + 4].opcode == OP_LONG
+	  && exp->elts[pc + 5].type == builtin_type_char
+	  && exp->language_defn->la_language == language_c)
+	{
+	  /* Attempt to print C character arrays using string syntax.
+	     Walk through the args, picking up one character from each
+	     of the OP_LONG expression elements.  If any array element
+	     does not match our expection of what we should find for
+	     a simple string, revert back to array printing.  Note that
+	     the last expression element is an explicit null terminator
+	     byte, which doesn't get printed. */
+	  tempstr = alloca (nargs);
+	  pc += 4;
+	  while (tem < nargs)
+	    {
+	      if (exp->elts[pc].opcode != OP_LONG
+		  || exp->elts[pc + 1].type != builtin_type_char)
+		{
+		  /* Not a simple array of char, use regular array printing. */
+		  tem = 0;
+		  break;
+		}
+	      else
+		{
+		  tempstr[tem++] =
+		    longest_to_int (exp->elts[pc + 2].longconst);
+		  pc += 4;
+		}
+	    }
+	}
+      if (tem > 0)
+	{
+	  LA_PRINT_STRING (stream, tempstr, nargs - 1, 0);
+	  (*pos) = pc;
+	}
+      else
+	{
+	  fputs_filtered (" {", stream);
+	  for (tem = 0; tem < nargs; tem++)
+	    {
+	      if (tem != 0)
+		{
+		  fputs_filtered (", ", stream);
+		}
+	      print_subexp (exp, pos, stream, PREC_ABOVE_COMMA);
+	    }
+	  fputs_filtered ("}", stream);
+	}
       return;
 
     case TERNOP_COND:
@@ -163,20 +230,20 @@ print_subexp (exp, pos, stream, prec)
       return;
 
     case STRUCTOP_STRUCT:
-      tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+      tem = longest_to_int (exp->elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (tem + 1);
       print_subexp (exp, pos, stream, PREC_SUFFIX);
       fputs_filtered (".", stream);
-      fputs_filtered (&exp->elts[pc + 1].string, stream);
+      fputs_filtered (&exp->elts[pc + 2].string, stream);
       return;
 
     /* Will not occur for Modula-2 */
     case STRUCTOP_PTR:
-      tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+      tem = longest_to_int (exp->elts[pc + 1].longconst);
+      (*pos) += 3 + BYTES_TO_EXP_ELEM (tem + 1);
       print_subexp (exp, pos, stream, PREC_SUFFIX);
       fputs_filtered ("->", stream);
-      fputs_filtered (&exp->elts[pc + 1].string, stream);
+      fputs_filtered (&exp->elts[pc + 2].string, stream);
       return;
 
     case BINOP_SUBSCRIPT:
@@ -256,7 +323,7 @@ print_subexp (exp, pos, stream, prec)
 
     /* Modula-2 ops */
 
-    case BINOP_MULTI_SUBSCRIPT:
+    case MULTI_SUBSCRIPT:
       (*pos) += 2;
       nargs = longest_to_int (exp->elts[pc + 1].longconst);
       print_subexp (exp, pos, stream, PREC_SUFFIX);
@@ -404,3 +471,134 @@ op_string(op)
       return op_print_tab[tem].string;
   return NULL;
 }
+
+#ifdef DEBUG_EXPRESSIONS
+
+/* Support for dumping the raw data from expressions in a human readable
+   form.  */
+
+void
+dump_expression (exp, stream, note)
+     struct expression *exp;
+     FILE *stream;
+     char *note;
+{
+  int elt;
+  char *opcode_name;
+  char *eltscan;
+  int eltsize;
+
+  fprintf_filtered (stream, "Dump of expression @ 0x%x, %s:\n", exp, note);
+  fprintf_filtered (stream, "\tLanguage %s, %d elements, %d bytes each.\n",
+		    exp->language_defn->la_name, exp -> nelts,
+		    sizeof (union exp_element));
+  fprintf_filtered (stream, "\t%5s  %20s  %16s  %s\n", "Index", "Opcode",
+		    "Hex Value", "String Value");
+  for (elt = 0; elt < exp -> nelts; elt++)
+    {
+      fprintf_filtered (stream, "\t%5d  ", elt);
+      switch (exp -> elts[elt].opcode)
+	{
+	  default: opcode_name = "<unknown>"; break;
+	  case OP_NULL: opcode_name = "OP_NULL"; break;
+	  case BINOP_ADD: opcode_name = "BINOP_ADD"; break;
+	  case BINOP_SUB: opcode_name = "BINOP_SUB"; break;
+	  case BINOP_MUL: opcode_name = "BINOP_MUL"; break;
+	  case BINOP_DIV: opcode_name = "BINOP_DIV"; break;
+	  case BINOP_REM: opcode_name = "BINOP_REM"; break;
+	  case BINOP_MOD: opcode_name = "BINOP_MOD"; break;
+	  case BINOP_LSH: opcode_name = "BINOP_LSH"; break;
+	  case BINOP_RSH: opcode_name = "BINOP_RSH"; break;
+	  case BINOP_LOGICAL_AND: opcode_name = "BINOP_LOGICAL_AND"; break;
+	  case BINOP_LOGICAL_OR: opcode_name = "BINOP_LOGICAL_OR"; break;
+	  case BINOP_BITWISE_AND: opcode_name = "BINOP_BITWISE_AND"; break;
+	  case BINOP_BITWISE_IOR: opcode_name = "BINOP_BITWISE_IOR"; break;
+	  case BINOP_BITWISE_XOR: opcode_name = "BINOP_BITWISE_XOR"; break;
+	  case BINOP_EQUAL: opcode_name = "BINOP_EQUAL"; break;
+	  case BINOP_NOTEQUAL: opcode_name = "BINOP_NOTEQUAL"; break;
+	  case BINOP_LESS: opcode_name = "BINOP_LESS"; break;
+	  case BINOP_GTR: opcode_name = "BINOP_GTR"; break;
+	  case BINOP_LEQ: opcode_name = "BINOP_LEQ"; break;
+	  case BINOP_GEQ: opcode_name = "BINOP_GEQ"; break;
+	  case BINOP_REPEAT: opcode_name = "BINOP_REPEAT"; break;
+	  case BINOP_ASSIGN: opcode_name = "BINOP_ASSIGN"; break;
+	  case BINOP_COMMA: opcode_name = "BINOP_COMMA"; break;
+	  case BINOP_SUBSCRIPT: opcode_name = "BINOP_SUBSCRIPT"; break;
+	  case MULTI_SUBSCRIPT: opcode_name = "MULTI_SUBSCRIPT"; break;
+	  case BINOP_EXP: opcode_name = "BINOP_EXP"; break;
+	  case BINOP_MIN: opcode_name = "BINOP_MIN"; break;
+	  case BINOP_MAX: opcode_name = "BINOP_MAX"; break;
+	  case BINOP_SCOPE: opcode_name = "BINOP_SCOPE"; break;
+	  case STRUCTOP_MEMBER: opcode_name = "STRUCTOP_MEMBER"; break;
+	  case STRUCTOP_MPTR: opcode_name = "STRUCTOP_MPTR"; break;
+	  case BINOP_INTDIV: opcode_name = "BINOP_INTDIV"; break;
+	  case BINOP_ASSIGN_MODIFY: opcode_name = "BINOP_ASSIGN_MODIFY"; break;
+	  case BINOP_VAL: opcode_name = "BINOP_VAL"; break;
+	  case BINOP_INCL: opcode_name = "BINOP_INCL"; break;
+	  case BINOP_EXCL: opcode_name = "BINOP_EXCL"; break;
+	  case BINOP_CONCAT: opcode_name = "BINOP_CONCAT"; break;
+	  case BINOP_END: opcode_name = "BINOP_END"; break;
+	  case TERNOP_COND: opcode_name = "TERNOP_COND"; break;
+	  case OP_LONG: opcode_name = "OP_LONG"; break;
+	  case OP_DOUBLE: opcode_name = "OP_DOUBLE"; break;
+	  case OP_VAR_VALUE: opcode_name = "OP_VAR_VALUE"; break;
+	  case OP_LAST: opcode_name = "OP_LAST"; break;
+	  case OP_REGISTER: opcode_name = "OP_REGISTER"; break;
+	  case OP_INTERNALVAR: opcode_name = "OP_INTERNALVAR"; break;
+	  case OP_FUNCALL: opcode_name = "OP_FUNCALL"; break;
+	  case OP_STRING: opcode_name = "OP_STRING"; break;
+	  case OP_BITSTRING: opcode_name = "OP_BITSTRING"; break;
+	  case OP_ARRAY: opcode_name = "OP_ARRAY"; break;
+	  case UNOP_CAST: opcode_name = "UNOP_CAST"; break;
+	  case UNOP_MEMVAL: opcode_name = "UNOP_MEMVAL"; break;
+	  case UNOP_NEG: opcode_name = "UNOP_NEG"; break;
+	  case UNOP_LOGICAL_NOT: opcode_name = "UNOP_LOGICAL_NOT"; break;
+	  case UNOP_COMPLEMENT: opcode_name = "UNOP_COMPLEMENT"; break;
+	  case UNOP_IND: opcode_name = "UNOP_IND"; break;
+	  case UNOP_ADDR: opcode_name = "UNOP_ADDR"; break;
+	  case UNOP_PREINCREMENT: opcode_name = "UNOP_PREINCREMENT"; break;
+	  case UNOP_POSTINCREMENT: opcode_name = "UNOP_POSTINCREMENT"; break;
+	  case UNOP_PREDECREMENT: opcode_name = "UNOP_PREDECREMENT"; break;
+	  case UNOP_POSTDECREMENT: opcode_name = "UNOP_POSTDECREMENT"; break;
+	  case UNOP_SIZEOF: opcode_name = "UNOP_SIZEOF"; break;
+	  case UNOP_PLUS: opcode_name = "UNOP_PLUS"; break;
+	  case UNOP_CAP: opcode_name = "UNOP_CAP"; break;
+	  case UNOP_CHR: opcode_name = "UNOP_CHR"; break;
+	  case UNOP_ORD: opcode_name = "UNOP_ORD"; break;
+	  case UNOP_ABS: opcode_name = "UNOP_ABS"; break;
+	  case UNOP_FLOAT: opcode_name = "UNOP_FLOAT"; break;
+	  case UNOP_HIGH: opcode_name = "UNOP_HIGH"; break;
+	  case UNOP_MAX: opcode_name = "UNOP_MAX"; break;
+	  case UNOP_MIN: opcode_name = "UNOP_MIN"; break;
+	  case UNOP_ODD: opcode_name = "UNOP_ODD"; break;
+	  case UNOP_TRUNC: opcode_name = "UNOP_TRUNC"; break;
+	  case OP_BOOL: opcode_name = "OP_BOOL"; break;
+	  case OP_M2_STRING: opcode_name = "OP_M2_STRING"; break;
+	  case STRUCTOP_STRUCT: opcode_name = "STRUCTOP_STRUCT"; break;
+	  case STRUCTOP_PTR: opcode_name = "STRUCTOP_PTR"; break;
+	  case OP_THIS: opcode_name = "OP_THIS"; break;
+	  case OP_SCOPE: opcode_name = "OP_SCOPE"; break;
+	  case OP_TYPE: opcode_name = "OP_TYPE"; break;
+	}
+      fprintf_filtered (stream, "%20s  ", opcode_name);
+      fprintf_filtered (stream,
+#if defined (LONG_LONG)
+			"%ll16x  ",
+#else
+			"%l16x  ",
+#endif
+			exp -> elts[elt].longconst);
+
+      for (eltscan = (char *) &exp->elts[elt],
+	     eltsize = sizeof (union exp_element) ;
+	   eltsize-- > 0;
+	   eltscan++)
+	{
+	  fprintf_filtered (stream, "%c",
+			    isprint (*eltscan) ? (*eltscan & 0xFF) : '.');
+	}
+      fprintf_filtered (stream, "\n");
+    }
+}
+
+#endif	/* DEBUG_EXPRESSIONS */
