@@ -148,7 +148,7 @@ static ExceptionHook oldExceptionHook;
 /* 									   */
 
 extern void
-return_to_prog PARAMS ((void));
+return_to_prog ();
 
 /* Restore the program's registers (including the stack pointer, which
    means we get the right stack and don't have to worry about popping our
@@ -550,7 +550,7 @@ char * parm;
 }
 
 /* Address of a routine to RTE to if we get a memory fault.  */
-static NORETURN void (*mem_fault_routine)() = NULL;
+static volatile void (*mem_fault_routine)() = NULL;
 
 /* Indicate to caller of mem2hex or hex2mem that there has been an
    error.  */
